#include <Pilot.h>
#include "soarDB.h"

/*****************************************************************************
 * OpenCreateDB - open a database. If it doesn't exist, create it.
 *****************************************************************************/
int OpenCreateDB(DmOpenRef *dbP, 
						UInt cardNo, 
						CharPtr name, 
						ULong dbcreator, 
						ULong type, 
						UInt mode)
{
	if ((*dbP=DmOpenDatabaseByTypeCreator(type,dbcreator,mode)) == 0) {
		DmCreateDatabase(cardNo,name,dbcreator,type,false);
		if ((*dbP=DmOpenDatabaseByTypeCreator(type,dbcreator,mode)) == 0) {
			return (DB_OPEN_FAIL);
		} else {
			return (DB_OPEN_CREATE);
		}
	}
	return (DB_OPEN_SUCCESS);
}

/*****************************************************************************
 * OpenDBCountRecords
 *****************************************************************************/
short OpenDBCountRecords(DmOpenRef	openDBDB)
{
	if (openDBDB == 0)
		return NO_RECORD;
	else
		return DmNumRecords(openDBDB);
}



/*****************************************************************************
 * OpenDBAddRecord - add a record in the open db at the start or
 *		end of the db (according to atStart)
 *****************************************************************************/
short OpenDBAddRecord(DmOpenRef		openDBDB,
						long		size,
						VoidPtr		openDBP,
						Boolean		atStart)
{
	VoidHand	recH;
	Byte		*recP;
	Err			err;
	UInt		dbIndex = 0;

	if ((dbIndex = OpenDBNewRecord(openDBDB, size, &recH, (VoidPtr)&recP, atStart))
			== NO_RECORD)
		return NO_RECORD;

	// write the data
	err = DmWrite(recP, 0, openDBP, size);

	// release record
	OpenDBReleaseRecord(openDBDB, dbIndex, &recH, (VoidPtr)&recP, !err);

	// delete on failure
	if (err) {
		OpenDBDeleteRecord(openDBDB, dbIndex);
		return NO_RECORD;
	}
	else
		return dbIndex;
}



/*****************************************************************************
 * OpenDBNewRecord - create a new record in the open db at the start or
 *		end of the db (according to atStart)
 *****************************************************************************/
short OpenDBNewRecord(DmOpenRef		openDBDB,
						long		size,
						VoidHand	*openDBH,
						VoidPtr		*openDBP,
						Boolean		atStart)
{
	UInt	dbIndex = 0;

	// check for an open db
 	if (openDBDB == 0)
		return false;

	if (atStart)
		dbIndex = 0;
	else
		dbIndex = DmNumRecords(openDBDB);

	*openDBP = NULL;

	// create and write a new record
	if ((*openDBH = DmNewRecord(openDBDB, &dbIndex, size)) == NULL ||
			(*openDBP = MemHandleLock(*openDBH)) == NULL) {

		// couldn't create properly, back out
		OpenDBReleaseRecord(openDBDB, dbIndex, openDBH, openDBP, false);

		return NO_RECORD;
	}

	return dbIndex;
}



/*****************************************************************************
 * OpenDBGetRecord - get a record from the OpenDB
 *****************************************************************************/
Boolean OpenDBGetRecord(DmOpenRef	openDBDB,
						int			index,
						VoidHand	*openDBH,
						VoidPtr		*openDBP)
{
	// init the ptr up front
	*openDBP = NULL;

	// check for an open db
 	if (openDBDB == 0)
		return false;

	// check for bad index
	if (index >= DmNumRecords(openDBDB))
		return false;

	// get the record
	if ((*openDBH = DmGetRecord(openDBDB, index)) == NULL ||
			(*openDBP = MemHandleLock(*openDBH)) == NULL) {

		// couldn't get properly, back out
		OpenDBReleaseRecord(openDBDB, index, openDBH, openDBP, false);

		return false;
	}

	return true;
}



/*****************************************************************************
 * OpenDBQueryRecord - get a read-only record from the OpenDB
 *****************************************************************************/
Boolean OpenDBQueryRecord(DmOpenRef	openDBDB,
						int			index,
						VoidHand	*openDBH,
						VoidPtr		*openDBP)
{
	// init the ptr up front
	*openDBP = NULL;

	// check for an open db
 	if (openDBDB == 0)
		return false;

	// check for bad index
	if (index >= DmNumRecords(openDBDB))
		return false;

	// get the record
	if ((*openDBH = DmQueryRecord(openDBDB, index)) == NULL ||
			(*openDBP = MemHandleLock(*openDBH)) == NULL) {

		// couldn't get properly, back out
		OpenDBReleaseRecord(openDBDB, index, openDBH, openDBP, false);

		return false;
	}

	return true;
}



/*****************************************************************************
 * OpenDBReleaseRecord - release a record from the openDB db.  
 *****************************************************************************/
Boolean OpenDBReleaseRecord(DmOpenRef	openDBDB,
							int			index,
							VoidHand	*openDBH,
							VoidPtr		*openDBP,
							Boolean		save)
{
	// check for an open db
 	if (openDBDB == 0)
		return false;

	// check for bad index
	if (index >= DmNumRecords(openDBDB))
		return false;

	// unlock first
	if (*openDBP)
		MemHandleUnlock(*openDBH);

	// release the record and close the database
	DmReleaseRecord(openDBDB, index, save);

	// clean up
	*openDBP = NULL;
	*openDBH = NULL;

	return true;
}



/*****************************************************************************
 * OpenDBDeleteRecord - delete a record from the openDB db.  
 *****************************************************************************/
Boolean OpenDBDeleteRecord(DmOpenRef	openDBDB,
							int			index)
{
	// check for an open db
 	if (openDBDB == 0)
		return false;

	// check for bad index and delete
	if (index >= DmNumRecords(openDBDB) ||
			DmRemoveRecord(openDBDB, index) != 0)
		return false;

	return true;
}



/*****************************************************************************
 * OpenDBEmpty - delete the records from the openDB db.  
 *****************************************************************************/
Boolean OpenDBEmpty(DmOpenRef	openDBDB)
{
	int		index;
	Boolean	result = true;

	// check for an open db
 	if (openDBDB == 0)
		return false;

	// delete backwards
	for (index = DmNumRecords(openDBDB) - 1; index > -1 && result; index--)
		if (DmRemoveRecord(openDBDB, index) != 0)
			result = false;

	return result;
}


/*****************************************************************************
 * OpenDBSetBackup
 *    set the bit in a database so the database will or will not be backed up
 *		by the default backup conduit
 * Parameters:
 *		DmOpenRef	dbRef			- reference to the OPEN database
 *		Boolean		setForBackup	- set for backup or not
 *****************************************************************************/
void OpenDBSetBackup(DmOpenRef	openDBRef,
					Boolean		setForBackup)
{
	UInt	cardNo,
			dbAttrs;
	LocalID dbID;

	// get the dbID then get the attributes
	DmOpenDatabaseInfo(openDBRef, &dbID, NULL, NULL, &cardNo, NULL);
	DmDatabaseInfo(cardNo, dbID, NULL, &dbAttrs,
			NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);

	// set the attribute
	if (setForBackup)
		// back up the database
		dbAttrs |= dmHdrAttrBackup;
	else
		// DON'T backup the database
		dbAttrs &= ~dmHdrAttrBackup;

	// save the attributes
	DmSetDatabaseInfo(cardNo, dbID, NULL, &dbAttrs,
			NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
}


/*****************************************************************************
 * OpenDBUpdateRecord - update a record.  NULL records are written out
 *	as empty
 *	Return: false if dbIndex out of range or write fails, true otherwise
 *****************************************************************************/
Boolean OpenDBUpdateRecord(DmOpenRef openDBDB,
						long  size,
						VoidPtr	openDBP,
						int	dbIndex)
{
	VoidHand	dbH = NULL;
	VoidPtr		dbP = NULL;
	Boolean		result = true;

	// fetch and write the record
	if ((UInt)dbIndex < DmNumRecords(openDBDB) &&
			(dbH = DmGetRecord(openDBDB, dbIndex)) != NULL &&
			(dbP = MemHandleLock(dbH)) != NULL) {
		if (openDBP)
			DmWrite(dbP, 0, openDBP, size);
		else
			// NULL data, so write 0s
			DmSet(dbP, 0, size, 0);
	}
	else
		result = false;

	// clean up
	if (dbP)
		MemHandleUnlock(dbH);
	if (dbH)
		DmReleaseRecord(openDBDB, dbIndex, result);

	return result;
}

